import { HandlerContext } from "$fresh/server.ts";
import { searchMovie } from "@lib/tmdb.ts";
import * as cache from "@lib/cache/cache.ts";

type CachedMovieQuery = {
  lastUpdated: number;
  data: unknown;
};

const CACHE_INTERVAL = 1000 * 60 * 24 * 30;

export const handler = async (
  _req: Request,
  _ctx: HandlerContext,
) => {
  const u = new URL(_req.url);

  const query = u.searchParams.get("q");

  if (!query) {
    return new Response("Bad Request", {
      status: 400,
    });
  }

  const headers = new Headers();
  headers.append("Content-Type", "application/json");

  const cacheId = `/movie/query/${query}`;

  const cachedResponse = await cache.get<CachedMovieQuery>(cacheId);
  if (
    cachedResponse && Date.now() < (cachedResponse.lastUpdated + CACHE_INTERVAL)
  ) {
    return new Response(JSON.stringify(cachedResponse.data), { headers });
  }

  const res = await searchMovie(query);

  cache.set(
    cacheId,
    JSON.stringify({
      lastUpdated: Date.now(),
      data: res,
    }),
  );

  return new Response(JSON.stringify(res.results));
};
